Package: CaDrA
Type: Package
Title: Candidate Driver Analysis
Version: 1.10.0
Authors@R: 
  c(person(given="Reina", family="Chau", role=c("aut","cre"), 
           email="rchau88@bu.edu", comment=c(ORCID="0000-0003-3012-1404")), 
    person(given="Katia", family="Bulekova", role=c("aut"), 
           email="ktrn@bu.edu", comment=c(ORCID="0000-0003-1560-2146")),
    person(given="Vinay", family="Kartha", role=c("aut"), 
           email="vkartha@bu.edu"), 
    person(given="Stefano", family="Monti", role=c("aut"), 
           email="smonti@bu.edu", comment=c(ORCID="0000-0002-9376-0660"))
   )
Description: Performs both stepwise and backward heuristic search for 
  candidate (epi)genetic drivers based on a binary multi-omics dataset. 
  CaDrA's main objective is to identify features which, together, are 
  significantly skewed or enriched pertaining to a given vector of 
  continuous scores (e.g. sample-specific scores representing a phenotypic 
  readout of interest, such as protein expression, pathway activity, etc.), 
  based on the union occurence (i.e. logical OR) of the events.
Depends: R (>= 4.4.0)
LazyData: false
License: GPL-3 + file LICENSE
URL: https://github.com/montilab/CaDrA/
Packaged: 2026-04-28 21:02:11 UTC; biocbuild
Encoding: UTF-8
RoxygenNote: 7.3.3
Imports: doParallel, ggplot2, gplots, graphics, grid, gtable, knnmi,
        MASS, methods, misc3d, plyr, ppcor, R.cache, reshape2, stats,
        SummarizedExperiment
Suggests: BiocManager, devtools, knitr, pheatmap, rmarkdown, testthat
        (>= 3.1.6)
Config/testthat/edition: 3
biocViews: Microarray, RNASeq, GeneExpression, Software,
        FeatureExtraction
VignetteBuilder: knitr
BugReports: https://github.com/montilab/CaDrA/issues
git_url: https://git.bioconductor.org/packages/CaDrA
git_branch: RELEASE_3_23
git_last_commit: 17d98f6
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: yes
Author: Reina Chau [aut, cre] (ORCID: <https://orcid.org/0000-0003-3012-1404>),
  Katia Bulekova [aut] (ORCID: <https://orcid.org/0000-0003-1560-2146>),
  Vinay Kartha [aut],
  Stefano Monti [aut] (ORCID: <https://orcid.org/0000-0002-9376-0660>)
Maintainer: Reina Chau <rchau88@bu.edu>
