#' Genomic Data from TCGA BRCA MUT + GISTIC
#'
#' A SummarizedExperiment object consists of 16,873 genomic features 
#' across 951 samples.
#'
#' @docType data
#'
#' @usage data(BRCA_GISTIC_MUT_SIG)
#'
#' @format An object of class \code{SummarizedExperiment} from 
#' \code{SummarizedExperiment} package
#' containing an assay of 16,873 rows (features) and 951 columns (samples)
#' see \code{SummarizedExperiment} for more details.
#' 
#' @return a SummarizedExperiment object
#' 
#' @references Kartha VK, Kern JG, Sebastiani P, Zhang L,
#' Varelas X, Monti S (2019) CaDrA: A computational framework for performing
#' candidate driver analyses using binary genomic features.
#' (\href{https://www.frontiersin.org/articles/10.3389/fgene.2019.00121/full}{Frontiers in Genetics})
#'
"BRCA_GISTIC_MUT_SIG"
