% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BRCA_GISTIC_MUT_SIG.R
\docType{data}
\name{BRCA_GISTIC_MUT_SIG}
\alias{BRCA_GISTIC_MUT_SIG}
\title{Genomic Data from TCGA BRCA MUT + GISTIC}
\format{
An object of class \code{SummarizedExperiment} from 
\code{SummarizedExperiment} package
containing an assay of 16,873 rows (features) and 951 columns (samples)
see \code{SummarizedExperiment} for more details.
}
\usage{
data(BRCA_GISTIC_MUT_SIG)
}
\value{
a SummarizedExperiment object
}
\description{
A SummarizedExperiment object consists of 16,873 genomic features 
across 951 samples.
}
\references{
Kartha VK, Kern JG, Sebastiani P, Zhang L,
Varelas X, Monti S (2019) CaDrA: A computational framework for performing
candidate driver analyses using binary genomic features.
(\href{https://www.frontiersin.org/articles/10.3389/fgene.2019.00121/full}{Frontiers in Genetics})
}
\keyword{datasets}
